"""
License:
This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""

from .image import Image


class Jpeg(Image):
    def __init__(self):
        super().__init__("jpeg")
